;; tape loader 

plus equ &0040
playcity equ &0041

loader_load_addr equ &a000
km_wait_char equ &bb06
txt_output equ &bb5a
scr_initialise equ &bbff

org loader_load_addr

exe_start:
call scr_initialise

ld e,14
ld hl,crtc_vals
ld bc,&bc00
set_crtc_vals:
out (c),c
inc b
ld a,(hl)
out (c),a
inc hl
dec b
inc c
dec e
jr nz,set_crtc_vals

;; display message
ld hl,gs_col_msg
call display_msg

;; detect playcity
call check_playcity
ld a,0
jr nz,noplaycity
ld hl,playcity_msg
call display_msg
ld a,1
noplaycity:
ld (playcity),a

call detect_plus
ld a,0
jr nz,noplus
ld hl,plus_msg
call display_msg

call plus_Unlock

ld a,1
noplus:
ld (plus),a

;; small pause
ld b,5
pause:
ld hl,0
pause2:
dec hl
ld a,h
or l
jr nz,pause2
djnz pause

di
;; clear screen
ld hl,&c000
ld e,l
ld d,h
inc de
ld (hl),0
ld bc,&3fff
ldir

;; mode 0
ld bc,&7f00+%10001100
out (c),c

ld a,(plus)
or a
jr z,no_plus_pal

ld hl,loadscr_plus_palette
call setup_plus_palette

jr do_load_scr

no_plus_pal:
ld hl,title_colours+16
call set_palette

do_load_scr:
;; load screen
ld ix,&c000
ld de,&4000
ld a,&ff
call topoload
jp nc,0

;; load main
defb &dd,&21
include "reloc.lod_s.asm"
defb &11
include "reloc.lod_l.asm"
ld a,&ff
call topoload
jp nc,0

di
ld a,(plus)
or a
jr z,no_fade


ld bc,&7fb8
out (c),c

ld de,&6400
call init_fade_out

ld b,16
fadeit:
push bc
ld b,&f5
fadeit2:
in a,(c)
rra
jr nc,fadeit2
fadeit2b:
in a,(c)
rra
jr c,fadeit2b

call update_fade_out

call update_fade_cols

pop bc
djnz fadeit
ld bc,&7fa0
out (c),c

call setup_plus_sprites
no_fade:

ld a,(plus)
defb &c3
include "reloc.lod_x.asm"



display_msg:
ld a,(hl)
and &7f
bit 7,(hl)
jp nz,txt_output
inc hl
call txt_output
jr display_msg

plus_msg:
defb 31,(20-(13/2)),13
defb "PLUS DETECTE","D"+&80

playcity_msg:
defb 31,(20-(17/2)),15
defb "PLAYCITY DETECTE","D"+&80

gs_col_msg:
defb 4,1
defb 29,0,0
defb 28,0,0,0
defb 28,1,26,26
defb 28,2,15,15
defb 28,3,25,25
defb 14,0
defb 15,1
defb 31,(20-(12/2)),6
defb "HEXAVIRUS v0"
defb 31,(20-(8/2)),10
defb 15,2
defb "LOADING..",&80


;; detect plus
detect_plus:
  ld    bc,&f782
  out    (c),c
  dec    b
  ld    a,&f
  out    (c),a
  inc    b
  out    (c),c
  dec    b
  in    c,(c)
  cp    c 
  ret    

;; colours for screen
title_colours:
include "gfx/bin/loadscrpal.asm"
defb &54 ;; border

crtc_vals:
defb &3f, 40, 46, &8e, &26, &00, 25, 30, &00, &07, &00,&00,&30,&00


include "pcdetect.asm"
include "fade.asm"
include "setup_plus.asm"



plus_Unlock:
ld hl,plus_Sequence
ld bc,&bc00+15
pu1:
ld a,(hl)
out (c),a
inc hl
dec c
jr nz,pu1
ld a,&cd
out (c),a
ld a,&ee
out (c),a
ret

plus_Sequence:
defb &ff, &0
defb &ff,&77,&b3,&51,&a8,&d4,&62,&39,&9c,&46,&2b,&15,&8a

set_palette:
ld bc,&7f00+16
set_colours2:
out (c),c
ld a,(hl)
out (c),a
dec hl
dec c
jp p,set_colours2
ret

include "topoload.asm"



end exe_start
