.module _cpc_DrawFastLine
.globl _cpc_DrawFastLine

;; mode 1
;; a x x x a x x x
;; x b x x x b x x
;; x x c x x x c x
;; x x x d x x x d

;; 0,1 = x1
;; 2,3 = y1
;; 4,5 = x2
;; 6,7 = y2

;; 8 = colour

;; if x2-x1 is positive we can draw forwards
;; and x1,y1 is the starting point
;; if it is negative x2,y2 is the starting point

.area _CODE
_cpc_DrawFastLine:
push ix
ld ix,#4
add ix,sp

;;------------------------------------------------
;; calc dx
ld l,4(ix)    	;; x2
ld h,5(ix)
ld e,0(ix)  	;; x1
ld d,1(ix)
or a
sbc hl,de   ;; dx = x2-x1

;; if negative, make positive.
jr p, cdf1

;; it's negative so make it positive.

xor a
sub l
ld l,a
sbc a,a
sub h
ld h,a


cdf1:
push hl

;;------------------------------------------------
;; decide draw origin

ld e,0(ix)		;; x1
ld d,1(ix)
ld l,2(ix)		;; y1
ld h,3(ix)
or a
jr z,cdf4

;; use x2,y2 as our draw origin because dx is negative

ld e,4(ix)		;; x2
ld d,5(ix)
ld l,6(ix)		;; y2
ld h,7(ix)
cdf4:
srl d			;; /2 because 2 pixels per byte
rr e
push de
call _cpc_GetScrAddrInternal
ld (scraddr),hl
pop hl

;; need to clean this up a bit...
ld a,l
and #0x1		;; pixel within byte
ld (pixel),a
;; and the mask for drawing
ld c,#0xaa
jr z,cdfl1
ld c,#0x55
cdfl1:
;; full colour pen
ld a,8(ix)
;; masked for putting onto screen
and c
;; store in colour
ld (cur_colour),a

;; mask, invert so we can isolate pixels when doing replace method
ld a,c
cpl
ld (cur_mask),a

;; restart colour is always left pixel
ld a,8(ix)
and #0x0aa
ld (res_colour),a

;; restart mask is always right pixel
ld a,#0x055
ld (res_mask),a


;;------------------------------------------------
;; calc dy
ld l,6(ix)	;; y2
ld h,7(ix)
ld e,2(ix)	;; y1
ld d,3(ix)
or a
sbc hl,de
ld a,0
jp p,cdf2

;; it's negative so make it positive
xor a
sub l
ld l,a
sbc a,a
sub h
ld h,a
ld a,1

cdf2:
ld (up_down_flag),a
pop de
ex de,hl
;; HL = dx
;; DE = dy

or a
sbc hl,de
add hl,de
sbc a,a
;; +ve means dx greater than dy
;; -ve means dy greater than dx
jr z,cdf3

;; switch roles of dx/dy
ex de,hl
;; HL = dy
;; DE = dx

cdf3:
;; HL = dx in terms of bresenham
;; DE = dy in terms of bresenham
ex de,hl
;; HL = dy in terms of bresenham
;; DE = dx in terms of bresenham

add hl,hl			;; HL = 2*dy in terms of bresenham
push hl
or a
sbc hl,de			;; 2*dy - dx
ld (_d),hl			;; d = 2*dy - dx
ex de,hl
;; HL = dx in terms of bresenham
;; DE = dy in terms of bresenham
add hl,hl			;; HL = 2*dx
ex de,hl
;; HL = dy in terms of bresenham
;; DE = dx in terms of bresenhams
or a
sbc hl,de			;; 2*dy-2*dx
ld (dy2subdx2),hl


ld l,6(ix)		;; y2
ld h,7(ix)
ld e,2(ix)		;; y1
ld d,3(ix)
or a
sbc hl,de 		;; dy = y2 - y1

add hl,hl
ld (dy2),hl		;; HL = 2*dy


ld hl,(dy2)
ld de,(dx)
or a
sbc hl,de		;; d = 2*dy - dx
ld (_d),hl

ld hl,(dy2) 
ld de,(dx2)
or a
sbc hl,de		;; ((2*dy)-(2*dx))
ld (dy2subdx2),hl

;; initial pixel
call plot_pixel
;; update x and counter
jr upd_x

more_pixels:
ld a,b
or c
jr z,drawLineDone

ld hl,(_d)
bit 7,h
jr nz,neg_or_zero
ld a,h
or l
jr z,neg_or_zero

call up_down_line

call plot_pixel

ld hl,(_d)
ld de,(dy2subdx2)
add hl,de
ld (_d),hl
jr upd_x

drawLineDone:
pop ix
ret


neg_or_zero:

call plot_pixel

ld hl,(_d)
ld de,(dy2)
add hl,de
ld (_d),hl



upd_x:
;; works out pixel and stuff
ld a,(pixel)
inc a
and #0x1
ld (pixel),a
or a
jr nz,more_pixels2

;; reset colour/mask
ld a,(res_colour)
ld (cur_colour),a
ld a,(res_mask)
ld (cur_mask),a

ld hl,(scraddr)
inc hl
ld (scraddr),hl

ld bc,(dx)
dec bc
ld (dx),bc

jr more_pixels

up_down_line:
ld hl,(scraddr)
call up_down_line_do
ld (scraddr),hl
ret

up_down_line_do:
ld a,(up_down_flag)
or a
jp  nz,_cpc_PrevLineInternal
jp _cpc_NextLineInternal 

more_pixels2:
;; shift...
ld a,(cur_colour)
rrca
ld (cur_colour),a

;; shift really
ld a,(cur_mask)
rrca
ld (cur_mask),a


ld bc,(dx)
dec bc
ld (dx),bc

jp more_pixels

plot_pixel:
ld hl,(scraddr)
ld a,(cur_colour)
ld c,a
call plot_pixel_replace
ret
  

;; C = colour
plot_pixel_replace:
ld a,(cur_mask)
and (hl)
or c
ld (hl),a
ret

;; C = colour
plot_pixel_xor:
ld a,(hl)
xor c
ld (hl),a
ret




.area _DATA
pixel:
.db 0
res_colour:
.db 0
res_mask:
.db 0
up_down_flag:
.db 0
;;left_right_flag:
;;.db 0
cur_colour:
.db 0
cur_mask:
.db 0
dy2subdx2:
.dw 0
;;dy2:
;;.dw 0
;;dy:
;;.dw 0
_d:
.dw 0
;;dx:
;;.dw 0
;;dx2:
;;.dw 0
scraddr:
.dw 0
